pro init_cal, gainchnls, tcalxx, tcalyy, freq, dpdf_guess, $
              calxx1, calxx0, calyy1, calyy0, $
              calxy1, calxy0, calyx1, calyx0, $
              cts_per_k_xx, cts_per_k_yy, $
              phase_zero, phase_slope, phase_cntr, phase_observed, $
              rf=rf
;+
;PURPOSE: Calculate cts_per+k, phase slope, and phase zero.
;CALLING SEQUENCE: 
;
;INIT_CAL, gainchnls, tcalxx, tcalyy, freq, dpdf_guess, $
;              calxx1, calxx0, calyy1, calyy0, $
;              calxy1, calxy0, calyx1, calyx0, $
;              cts_per_k_xx, cts_per_k_yy, $
;              phase_zero, phase_slope, phase_cntr, phase_observed, $
;              rf=rf
;
;INPUTS: 
;GAINCHNLS, the vector of channels to use in deriving the output
;       parameters.  
;TCALXX, TCALYY, the equivalent temperature of the cals
;FREQ, the array of freqs for GAINCHNLS. freqs are RF freqs if RF is
;       set, otherwise freqs are baseband freqs. BEST TO SET RF.
;DPDF_GUESS, the guessed vaue for the phase slope (rad/MHz)
;CALXX1[nchan, nr], the array of cal-on xx product spectra
;CALXX0[nchan, nr], the array of cal-off xx product spectra
;CALYY1[nchan, nr], the array of cal-on yy product spectra
;CALYY0[nchan, nr], the array of cal-off yy product spectra
;
;CALXY1[nchan, nr], the array of cal-on xy product spectra
;CALXY0[nchan, nr], the array of cal-off xy product spectra
;CALYX1[nchan, nr], the array of cal-on yx product spectra
;CALYX0[nchan, nr], the array of cal-off yx product spectra
;
;OUTPUTS:
;CTS_PER_K_XX, CTS_PER_K_YY, the counts/K for the x and y signal paths.
;PHASE_ZERO, the phase difference between X and Y signal paths 
;       for freq=mean freq of GAINCHNLS (radians).
;PHASE_SLOPE, the phase slope in radians/MHz.
;PHASE_CNTR, the phase of the central freq array (radians)
;phase_observed[nchan,nr], the array of observed phases
;-

sz= size( calxx1)
nr_cal= sz[ 2]
nchnls= sz[ 1]

; INITIALIZE THE COUNTS_PER_K FOR EACH CAL...
   cts_per_k_xx= fltarr( nr_cal)
   cts_per_k_yy= fltarr( nr_cal)

;INITIALIZE THE PHASE ZERO AND SLOPE, and their uncertainties, for the
; XY products...
phase_zero= fltarr( nr_cal, 2)
phase_slope= fltarr( nr_cal, 2)
phase_cntr= fltarr( nr_cal)
phase_observed= fltarr( nchnls, nr_cal, 2)

calxysw_phasecorr= fltarr( nchnls, nr_cal)
calyxsw_phasecorr= fltarr( nchnls, nr_cal)

;define the cal deflection...
calxxsw= calxx1- calxx0
calyysw= calxx1- calyy0
calxysw= calxy1- calxy0
calyxsw= calyx1- calyx0

;WE GO THROUGH EACH CALON/OFF PAIR
   for nc=0, nr_cal-1 do begin
      
;DETERMINE THE COUNTS PER KELVIN...
intensitycal_self, calxx1[ *,nc], calxx0[ *,nc], $
             gainchnls, tcalxx, counts_per_k_xx
      cts_per_k_xx[ nc]= counts_per_k_xx
intensitycal_self, calyy1[ *,nc], calyy0[ *,nc], $
             gainchnls, tcalyy, counts_per_k_yy
      cts_per_k_yy[ nc]= counts_per_k_yy

; LEAST-SQUARES FIT FOR THE PHASE OF THE CORRELATED CAL AS A FUNCTION
; OF FREQUENCY; WE RETURN THE PHASE SLOPE AND OFFSET
      phasecal_cross, calxysw[ *,nc], calyxsw[ *,nc], $
                      gainchnls, dpdf_guess, freq, $
                      ozero, oslope, phase_obs, $
                      SIGMA_PHASEFIT=sigma_phasefit, rf=rf
      phase_zero[ nc, *]= ozero
      phase_slope[ nc, *]= oslope
      freq_cntr= freq[ n_elements(freq)/2 - 1]
      phase_cntr[nc]= ozero[0]+ oslope[0]* freq_cntr
      phase_observed[ *, nc]= phase_obs
;stop

;CHECK RESULTS MANUALLY...
phaseplot_08, 'Cal defln phase (rad) vs freq nc=' + strtrim(nc,2), $
        freq, calxysw[*,nc], calyxsw[ *,nc], $
        ozero, oslope
;stop
wait, 0.1
phasecorr_xyyx, calxysw[*,nc], calyxsw[*,nc], freq, ozero, oslope, $
                calxy_phasecorr, calyx_phasecorr
calxysw_phasecorr[ *,nc]= calxy_phasecorr
calyxsw_phasecorr[ *,nc]= calyx_phasecorr
   endfor

end
